from django.conf import settings
from django.urls import reverse
from django.views.generic.base import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from ..view_mixins import EdcLabelViewMixin


class HomeView(EdcViewMixin, NavbarViewMixin, EdcLabelViewMixin, TemplateView):

    template_name = f"edc_label/bootstrap{settings.EDC_BOOTSTRAP}/home.html"
    navbar_name = "edc_label"
    navbar_selected_item = "label"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        printer_setup_url = reverse("edc_label:printer_setup_url")
        context.update(
            printer_setup_url=printer_setup_url,
        )
        return context



from django.test import TestCase

from ...printer import Printer


class DummyCupsConnection:

    printer_name = "test_printer"

    properties = {"printer-state-reasons": [""]}

    def getPrinters(self):
        return {self.printer_name: self.properties}

    def createJob(self, *args):
        return 1

    def startDocument(self, *args):
        return 1

    def writeRequestData(self, *args):
        return 1

    def finishDocument(self, *args):
        return 1


class TestLabels(TestCase):
    def test_dummy(self):
        connection = DummyCupsConnection()
        connection.getPrinters().get("test_printer")

    def test_str(self):
        printer = Printer(name="test_printer", print_server_func=DummyCupsConnection)
        self.assertTrue(str(printer))

    def test_repr(self):
        printer = Printer(name="test_printer", print_server_func=DummyCupsConnection)
        self.assertTrue(repr(printer))

    def test_stream_job(self):
        printer = Printer(name="test_printer", print_server_func=DummyCupsConnection)
        zpl_data = {}
        jobid = printer.stream_print(zpl_data)
        self.assertIsNotNone(jobid)



import socket

from cups import Connection, IPPError
from django.apps import apps as django_apps
from django.conf import settings
from django.utils.translation import gettext as _

from .constants import (
    CLINIC_LABEL_PRINTER_NAME,
    LAB_LABEL_PRINTER_NAME,
    PRINT_SERVER_NAME,
)
from .printer import Printer


class PrinterError(Exception):
    pass


class PrintServerError(Exception):
    pass


class PrintersMixin:
    @property
    def connect_cls(self):
        return Connection

    @property
    def user_profile(self):
        UserProfile = django_apps.get_model("edc_auth.userprofile")
        return UserProfile.objects.get(user=self.request.user)

    @property
    def print_server_name(self):
        """Returns a string."""
        return self.request.session.get(PRINT_SERVER_NAME, self.user_profile.print_server)

    @property
    def clinic_label_printer_name(self):
        """Returns a string."""
        return self.request.session.get(
            CLINIC_LABEL_PRINTER_NAME, self.user_profile.clinic_label_printer
        )

    @property
    def lab_label_printer_name(self):
        """Returns a string."""
        return self.request.session.get(
            LAB_LABEL_PRINTER_NAME, self.user_profile.lab_label_printer
        )

    @property
    def print_server_ip(self):
        if self.print_server_name == "localhost":
            return None
        try:
            return socket.gethostbyname(self.print_server_name)
        except (TypeError, socket.gaierror):
            return self.print_server_name

    @property
    def print_servers(self):
        """Returns a list of CUPS print servers.

        Default is ['localhost'].
        """
        return getattr(settings, "CUPS_SERVERS", ["localhost"])

    def print_server(self):
        """Returns a CUPS connection."""
        cups_connection = None
        if self.print_server_name:
            try:
                if not self.print_server_ip:
                    cups_connection = self.connect_cls()
                else:
                    cups_connection = self.connect_cls(self.print_server_ip)
            except (RuntimeError, IPPError) as e:
                raise PrintServerError(
                    f"{_('Unable to connect to print server. Tried ')}"
                    f"'{self.print_server_name}'. {_('Got')} {e}"
                )
        else:
            raise PrintServerError(_("Print server not defined"))
        return cups_connection

    @property
    def printers(self):
        """Returns a mapping of PrinterProperties objects
        or an empty mapping.

        If print server is not defined, raises.
        """
        printers = {}
        cups_printers = self.print_server().getPrinters()
        for name in cups_printers:
            printer = Printer(
                name=name,
                print_server_func=self.print_server,
                print_server_name=self.print_server_name,
                print_server_ip=self.print_server_ip,
            )
            printers.update({name: printer})
        return printers

    def get_label_printer(self, name):
        """Returns a PrinterProperties object, None or raises."""
        printer = self.printers.get(name)
        if not printer:
            raise PrinterError(
                f"{_('Printer does not exist. Got')} {name}. "
                f"{_('Installed printers are')} {list(self.printers)}."
            )
        return printer

    @property
    def clinic_label_printer(self):
        """Returns a PrinterProperties object or None."""
        return self.get_label_printer(self.clinic_label_printer_name)

    @property
    def lab_label_printer(self):
        """Returns a PrinterProperties object or None."""
        return self.get_label_printer(self.lab_label_printer_name)



from django.contrib import messages
from django.views.generic.base import ContextMixin

from .printers_mixin import PrinterError, PrintersMixin, PrintServerError


class EdcLabelViewMixin(PrintersMixin, ContextMixin):

    error_messages = []

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(print_servers=self.print_servers)
        try:
            context.update(selected_print_server_name=self.print_server_name)
        except PrintServerError as e:
            messages.error(self.request, str(e))
        else:
            try:
                context.update(printers=self.printers)
            except (PrintServerError, PrinterError) as e:
                messages.error(self.request, str(e))
            else:
                try:
                    context.update(selected_clinic_label_printer=self.clinic_label_printer)
                except PrinterError:
                    pass
                try:
                    context.update(selected_lab_label_printer=self.lab_label_printer)
                except PrinterError:
                    pass
        error_messages = list(set(self.error_messages))
        for message in error_messages:
            messages.error(self.request, message)
        return context



from django.contrib import messages

from .job_result import JobResult
from .printers_mixin import PrinterError, PrintersMixin, PrintServerError


class LabPrintersMixin(PrintersMixin):

    label_cls = None
    job_result_cls = JobResult

    @property
    def printer(self):
        try:
            printer = self.lab_label_printer
        except PrinterError:
            messages.error(
                self.request,
                'Your "lab" label printer is not configured. '
                "See Edc Label in Administration.",
            )
            raise PrinterError("lab_label_printer not set. Got None")
        except PrintServerError as e:
            messages.error(self.request, str(e))
            raise PrinterError(e)
        return printer

    def print_labels(self, pks=None, request=None):
        """Returns a job_result object or None after printing."""
        zpl_data = b""
        try:
            printer = self.printer
        except PrinterError:
            printer = None
            job_id = None
            job_result = None
        else:
            for pk in pks:
                label = self.label_cls(pk=pk, children_count=len(pks), request=request)
                zpl_data += label.render_as_zpl_data()
            job_id = printer.stream_print(zpl_data=zpl_data)
            job_result = self.job_result_cls(
                name=self.label_cls.label_template_name,
                copies=1,
                job_ids=[job_id],
                printer=printer,
            )
        return job_result



from django.contrib.auth.mixins import LoginRequiredMixin
from django.http.response import HttpResponseRedirect
from django.urls.base import reverse
from django.views.generic.edit import ProcessFormView
from edc_auth.models import UserProfile

from ..constants import (
    CLINIC_LABEL_PRINTER_NAME,
    LAB_LABEL_PRINTER_NAME,
    PRINT_SERVER_NAME,
)


class ChangePrinterView(LoginRequiredMixin, ProcessFormView):

    success_url = "edc_label:home_url"
    empty_selection = "--"

    def post(self, request, *args, **kwargs):

        user_profile = UserProfile.objects.get(user=self.request.user)

        print_server_name = request.POST.get(PRINT_SERVER_NAME)
        if print_server_name:
            if print_server_name == self.empty_selection:
                print_server_name = None
            request.session[PRINT_SERVER_NAME] = print_server_name
            user_profile.print_server = print_server_name

        clinic_label_printer_name = request.POST.get(CLINIC_LABEL_PRINTER_NAME)
        if clinic_label_printer_name:
            if clinic_label_printer_name == self.empty_selection:
                clinic_label_printer_name = None
            request.session[CLINIC_LABEL_PRINTER_NAME] = clinic_label_printer_name
            user_profile.clinic_label_printer = clinic_label_printer_name

        lab_label_printer_name = request.POST.get(LAB_LABEL_PRINTER_NAME)
        if lab_label_printer_name:
            if lab_label_printer_name == self.empty_selection:
                lab_label_printer_name = None
            request.session[LAB_LABEL_PRINTER_NAME] = lab_label_printer_name
            user_profile.lab_label_printer = lab_label_printer_name

        user_profile.save()
        success_url = reverse(self.success_url)

        return HttpResponseRedirect(redirect_to=success_url)



from .job_result import JobResult, add_job_results_to_messages
from .lab_printers_mixin import LabPrintersMixin
from .label import Label
from .printers_mixin import PrinterError, PrintersMixin, PrintServerError
from .view_mixins import EdcLabelViewMixin



from django.conf import settings
from django.views.generic import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin


class BrowserPrintLabelsView(EdcViewMixin, NavbarViewMixin, TemplateView):

    template_name = "edc_label/browser_print_labels.html"
    navbar_name = "edc_label"
    navbar_selected_item = "label"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        browser_print_page_auto_back = getattr(
            settings, "EDC_LABEL_BROWSER_PRINT_PAGE_AUTO_BACK", True
        )
        context.update(browser_print_page_auto_back=browser_print_page_auto_back)
        return context



from django.conf import settings
from django.contrib import admin
from django.urls import include, path, re_path

from edc_label.views.print_setup_view import PrinterSetupView

from .views import BrowserPrintLabelsView, ChangePrinterView, HomeView, PrintLabelView

app_name = "edc_label"

urlpatterns = [
    path(
        r"browser_print_labels",
        BrowserPrintLabelsView.as_view(),
        name="browser_print_labels_url",
    ),
    re_path(
        r"printer/change/(?P<printer_type>\w+)/",
        ChangePrinterView.as_view(),
        name="change_session_printer",
    ),
    re_path(
        r"print/label/(?P<printer_name>\w+)/(?P<label_template_name>)\w+/",
        PrintLabelView.as_view(),
        name="print_label",
    ),
    path("print/label/", PrintLabelView.as_view(), name="print_label"),
    path(
        "print_server/change/",
        ChangePrinterView.as_view(),
        name="change_session_print_server",
    ),
    re_path(
        r"print/(?P<label_name>\w+)/"
        r"(?P<copies>\d+)/(?P<app_label>\w+)/"
        r"(?P<model_name>\w+)/"
        r"(?P<pk>[a-f0-9]{8}-?[a-f0-9]{4}-?4[a-f0-9]{3}-?"
        r"[89ab][a-f0-9]{3}-?[a-f0-9]{12})/$",
        HomeView.as_view(),
        name="print-test-label",
    ),
    re_path(r"print/(?P<label_name>\w+)/$", HomeView.as_view(), name="print-test-label"),
    path("printer_setup/", PrinterSetupView.as_view(), name="printer_setup_url"),
    path("", HomeView.as_view(), name="home_url"),
]

if settings.APP_NAME == "edc_label":
    url_patterns = urlpatterns + [
        path("accounts/", include("edc_auth.urls")),
        path("admin/", admin.site.urls),
    ]



import os
import tempfile
from uuid import uuid4

import cups


class PrinterError(Exception):
    pass


class Printer:
    """A simple wrapper of a CUPS printer description.

    The CUPS printer description is a value from the mapping
    returned by getPrinters()

    * Replaces "-" with "_" in key names
    * Adds extra properties and methods.
    """

    def __init__(
        self,
        name=None,
        print_server_func=None,
        print_server_ip=None,
        print_server_name=None,
    ):
        self.name = name
        self.printer_info = None
        self.printer_make_and_model = None
        self.print_server = print_server_func
        self.print_server_name = print_server_name
        self.print_server_ip = print_server_ip
        cups_connection = print_server_func()
        try:
            cups_properties = cups_connection.getPrinters().get(name)
        except cups.IPPError:
            pass
        else:
            for k, v in cups_properties.items():
                k = k.replace("-", "_")
                setattr(self, k, v)
        self.printer_state_reasons = [r for r in self.printer_state_reasons if r != "none"]
        if self.printer_state_reasons:
            self.printer_state_reasons = ", ".join(self.printer_state_reasons)

    def __str__(self):
        return f"{self.printer_info or self.name} ({self.printer_make_and_model})"

    def __repr__(self):
        return f"{self.__class__}(name={self.name})"

    def stream_print(self, zpl_data=None):
        """Returns a job_id after sending zpl_data
        to the printer.
        """
        cups_connection = self.print_server()
        try:
            job_id = cups_connection.createJob(self.name, "", {})
            cups_connection.startDocument(
                self.name, job_id, str(uuid4()), cups.CUPS_FORMAT_RAW, 1
            )
            cups_connection.writeRequestData(zpl_data, len(zpl_data))
            cups_connection.finishDocument(self.name)
        except cups.IPPError as e:
            raise PrinterError(f"Failed to print. Tried printing to '{self.name}'. Got '{e}'.")
        return job_id

    def print_file(self, zpl_data=None):
        """Returns a job_id after sending zpl_data
        to the printer (uses printFile).
        """
        job_id = None
        _, temp = tempfile.mkstemp()
        cups_connection = self.print_server()
        try:
            with open(temp, "w") as f:
                f.write(zpl_data)
            #  note: "raw" attr is to prevent CUPS from rendering
            args = (self.name, temp, "edc_label", {"raw": temp})
            try:
                job_id = cups_connection.printFile(*args)
            except cups.IPPError as e:
                raise PrinterError(f"{e} using options {args}.")
        finally:
            os.remove(temp)
        return job_id



from django.apps import apps as django_apps

app_name = "edc_label"
LABELING = "LABELING"

codenames = []
for app_config in django_apps.get_app_configs():
    if app_config.name in [
        app_name,
    ]:
        for model_cls in app_config.get_models():
            app_name, model_name = model_cls._meta.label_lower.split(".")
            for prefix in ["add", "change", "view", "delete"]:
                codenames.append(f"{app_name}.{prefix}_{model_name}")
codenames.sort()



# Generated by Django 3.2.7 on 2021-10-12 04:05

import _socket
from django.db import migrations, models
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ZplLabelTemplates',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, editable=False, help_text='System auto field. UUID primary key.', primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('zpl_data', models.TextField(max_length=1000)),
            ],
            options={
                'verbose_name': 'Label template',
                'verbose_name_plural': 'Label templates',
                'ordering': ('-modified', '-created'),
                'get_latest_by': 'modified',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'export', 'import'),
            },
        ),
    ]
